import { v4 as uuidv4 } from 'uuid';
import fs, { createReadStream } from 'node:fs';
import path from 'node:path';
import fetch from 'node-fetch';
import express from 'express';
import axios from 'axios';
import { fileFromPath } from 'formdata-node/file-from-path';
import * as Misskey from 'misskey-js';

const __dirname = path.resolve();
const config = JSON.parse(fs.readFileSync(path.resolve("config", "config.json")));
const token_file = JSON.parse(fs.readFileSync(path.resolve("config", "token.json")));

let app = express();
let port = process.env.PORT || 4000;

let host = config.host;

let app_name = config.app.name;
let app_callback = config.app.callback;
let app_permission = config.app.permission;

// https://misskey.io/miauth/c1f6d42b-468b-4fd2-8274-e58abdedef6f?name=MyApp&callback=https%3A%2F%2Fmyapp.example.com%2Fcallback&permission=write:notes,write:following,read:drive

let session_id = uuidv4();
let auth_url = `${host}/miauth/${session_id}?name=${app_name}&callback=${app_callback}&permission=${app_permission}`;
let auth_status = 0;

console.log(auth_url);

app.get('/callback', function(req, res) {
    res.send({
        "status": 200,
        "success": true,
    })

    auth_status = 1;
});

async function check_auth_status() {
    return await new Promise(resolve => {
        const check = setInterval(() => {
            console.log("Checking auth status...");
    
            if(auth_status == 1) {
                console.log("Bot has been authenticated");
                resolve(true);
                clearInterval(check);
                return true;
            }
        }, 2000);
    });
};

async function get_access_token(host, session_id) {
    const response = await fetch(`${host}/api/miauth/${session_id}/check`, {
        method: 'POST'
    });
    
    const data = await response.json();
    console.log(data);

    const res_status = data.ok;

    if(data.ok == true) {
        return data.token;
    } else {
        return false; // error occurred 
    }
};

async function create_image(access_token, file_path) {
    axios.post(`${host}/api/drive/files/create`, {
        file: fs.createReadStream(file_path)
    }, {
        headers: {
            'Content-Type': 'multipart/form-data',
            'Authorization': `Bearer ${access_token}`
        }
    }).then((data) => {
        return data;
    });

    // return info:
    // https://misskey.winscloud.net/api-doc#tag/drive/operation/drive___files___create
};

(async () => {
    let access_token = false;

    if(!token_file.token) { // if existing token not exist
        await check_auth_status()
        access_token = await get_access_token(host, session_id);
        
        let json_data = {
            "token": access_token
        }
        
        fs.writeFileSync(
            path.resolve("config", "token.json"),
            JSON.stringify(json_data)
        );
    } else {
        access_token = token_file.token;
    }

    const cli = new Misskey.api.APIClient({
        origin: host,
        credential: access_token
    });

    let static_image_path = path.join(__dirname, '/data/current_radar.jpg');
    create_image(access_token, static_image_path);
})();

app.listen(port);
console.log(`Callback URL: http://localhost:${port}/callback`);